"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TlvCodec = exports.TlvTag = exports.TlvType = void 0;
const LEBufferReader_1 = require("../util/LEBufferReader");
const LEBufferWriter_1 = require("../util/LEBufferWriter");
var TlvType;
(function (TlvType) {
    TlvType[TlvType["SignedInt"] = 0] = "SignedInt";
    TlvType[TlvType["UnsignedInt"] = 4] = "UnsignedInt";
    TlvType[TlvType["Boolean"] = 8] = "Boolean";
    TlvType[TlvType["Float"] = 10] = "Float";
    TlvType[TlvType["Double"] = 11] = "Double";
    TlvType[TlvType["String"] = 12] = "String";
    TlvType[TlvType["ByteString"] = 16] = "ByteString";
    TlvType[TlvType["Null"] = 20] = "Null";
    TlvType[TlvType["Structure"] = 21] = "Structure";
    TlvType[TlvType["Array"] = 22] = "Array";
    TlvType[TlvType["List"] = 23] = "List";
    TlvType[TlvType["EndOfContainer"] = 24] = "EndOfContainer";
})(TlvType = exports.TlvType || (exports.TlvType = {}));
const COMMON_PROFILE = 0x00000000;
const UNSPECIFIED_PROFILE = 0xFFFFFFFF;
class TlvTag {
    constructor(profile, id) {
        this.profile = profile;
        this.id = id;
    }
    toNumber() {
        if (this.profile === UNSPECIFIED_PROFILE)
            return this.id;
        return this.profile << 64 + this.id;
    }
    toString() {
        if (this.profile === UNSPECIFIED_PROFILE) {
            if (this.id === 0xFFFFFFFF) {
                return "TagAnonymous";
            }
            else {
                return `TagContextSpecific(${this.id})`;
            }
        }
        else if (this.profile === COMMON_PROFILE) {
            return `TagCommon(${this.id})`;
        }
        else {
            return `TagVendor(${this.profile}, ${this.id})`;
        }
    }
    equals({ profile, id }) {
        return this.profile === profile && this.id === id;
    }
    isAnonymous() {
        return this.equals(TlvTag.Anonymous);
    }
    isContextual() {
        return this.profile === UNSPECIFIED_PROFILE;
    }
    isCommon() {
        return this.profile === COMMON_PROFILE;
    }
}
exports.TlvTag = TlvTag;
TlvTag.Anonymous = new TlvTag(UNSPECIFIED_PROFILE, 0xFFFFFFFF);
TlvTag.contextual = (id) => new TlvTag(UNSPECIFIED_PROFILE, id);
class TlvCodec {
    static decodeElement(data) {
        return this.decodeElementInternal(new LEBufferReader_1.LEBufferReader(data));
    }
    static decodeElementInternal(reader) {
        const { tag, typeSizeByte } = this.decodeTagAndTypeSize(reader);
        const { value, type } = this.decodeValue(reader, typeSizeByte);
        return { tag, type, value };
    }
    static decodeTagAndTypeSize(reader) {
        const controlByte = reader.readUInt8();
        const tag = this.decodeTag(reader, (controlByte & 0xE0));
        return { tag, typeSizeByte: controlByte & 0x1F };
    }
    static decodeTag(reader, tagControl) {
        switch (tagControl) {
            case 0:
                return TlvTag.Anonymous;
            case 32:
                return TlvTag.contextual(reader.readUInt8());
            case 64:
                return TlvTag.contextual(reader.readUInt16());
            case 96:
                return TlvTag.contextual(reader.readUInt32());
            case 128:
            case 160:
                throw new Error(`Unsupported implicit profile ${tagControl}`);
            case 192:
                return new TlvTag(reader.readUInt32(), reader.readUInt16());
            case 192:
                return new TlvTag(reader.readUInt32(), reader.readUInt32());
            default:
                throw new Error(`Unexpected tagControl ${tagControl}`);
        }
    }
    static decodeValue(reader, typeSizeByte) {
        switch (typeSizeByte) {
            case TlvType.SignedInt + 0:
                return { value: reader.readInt8(), type: TlvType.SignedInt };
            case TlvType.SignedInt + 1:
                return { value: reader.readInt16(), type: TlvType.SignedInt };
            case TlvType.SignedInt + 2:
                return { value: reader.readInt32(), type: TlvType.SignedInt };
            case TlvType.SignedInt + 3:
                return { value: reader.readInt64(), type: TlvType.SignedInt };
            case TlvType.UnsignedInt + 0:
                return { value: reader.readUInt8(), type: TlvType.UnsignedInt };
            case TlvType.UnsignedInt + 1:
                return { value: reader.readUInt16(), type: TlvType.UnsignedInt };
            case TlvType.UnsignedInt + 2:
                return { value: reader.readUInt32(), type: TlvType.UnsignedInt };
            case TlvType.UnsignedInt + 3:
                return { value: reader.readUInt64(), type: TlvType.UnsignedInt };
            case TlvType.Boolean:
                return { value: false, type: TlvType.Boolean };
            case TlvType.Boolean + 1:
                return { value: true, type: TlvType.Boolean };
            case TlvType.Float:
                return { value: reader.readFloat(), type: TlvType.Float };
            case TlvType.Double:
                return { value: reader.readDouble(), type: TlvType.Double };
            case TlvType.String + 0:
                return { value: reader.readString(reader.readUInt8()), type: TlvType.String };
            case TlvType.String + 1:
                return { value: reader.readString(reader.readUInt16()), type: TlvType.String };
            case TlvType.String + 2:
                return { value: reader.readString(reader.readUInt32()), type: TlvType.String };
            case TlvType.String + 3:
                return { value: reader.readString(Number(reader.readUInt64())), type: TlvType.String };
            case TlvType.ByteString + 0:
                return { value: reader.readBytes(reader.readUInt8()), type: TlvType.ByteString };
            case TlvType.ByteString + 1:
                return { value: reader.readBytes(reader.readUInt16()), type: TlvType.ByteString };
            case TlvType.ByteString + 2:
                return { value: reader.readBytes(reader.readUInt32()), type: TlvType.ByteString };
            case TlvType.ByteString + 3:
                return { value: reader.readBytes(Number(reader.readUInt64())), type: TlvType.ByteString };
            case TlvType.Null:
                return { value: null, type: TlvType.Null };
            case TlvType.Array:
                return { value: this.decodeContainer(reader), type: TlvType.Array };
            case TlvType.List:
                return { value: this.decodeContainer(reader), type: TlvType.List };
            case TlvType.Structure:
                return { value: this.decodeContainer(reader), type: TlvType.Structure };
            case TlvType.EndOfContainer:
                return { value: undefined, type: TlvType.EndOfContainer };
            default:
                throw new Error(`Unexpected elementType 0x${typeSizeByte.toFixed(16)}`);
        }
    }
    static decodeContainer(reader) {
        const result = new Array();
        while (true) {
            const element = this.decodeElementInternal(reader);
            if (element.type === TlvType.EndOfContainer)
                return result;
            result.push(element);
        }
    }
    static encodeElement(element) {
        const writer = new LEBufferWriter_1.LEBufferWriter();
        this.encodeElementInternal(writer, element);
        return writer.toBuffer();
    }
    static encodeElementInternal(writer, element) {
        if (element === undefined)
            return;
        switch (element.type) {
            case TlvType.ByteString:
                return this.encodeByteString(writer, element);
            case TlvType.String:
                return this.encodeString(writer, element);
            case TlvType.UnsignedInt:
                return this.encodeUnsignedInt(writer, element);
            case TlvType.SignedInt:
                return this.encodeSignedInt(writer, element);
            case TlvType.Boolean:
                return this.encodeBoolean(writer, element);
            case TlvType.Structure:
                return this.encodeContainer(writer, element);
            case TlvType.Array:
                return this.encodeContainer(writer, element);
            case TlvType.List:
                return this.encodeContainer(writer, element);
            default:
                throw new Error(`Unsupported element type: ${element.type}`);
        }
    }
    static encodeString(writer, { value, tag }) {
        const length = value.length;
        const size = this.getUnsignedIntSize(length);
        this.encodeControlByteAndTag(writer, TlvType.String | size, tag);
        this.encodeUnsignedIntBytes(writer, length, size);
        writer.writeString(value);
    }
    static encodeByteString(writer, { value, tag }) {
        const length = value.length;
        const size = this.getUnsignedIntSize(length);
        this.encodeControlByteAndTag(writer, TlvType.ByteString | size, tag);
        this.encodeUnsignedIntBytes(writer, length, size);
        writer.writeBytes(value);
    }
    static encodeUnsignedInt(writer, { value, tag }) {
        const size = this.getUnsignedIntSize(value);
        this.encodeControlByteAndTag(writer, TlvType.UnsignedInt | size, tag);
        this.encodeUnsignedIntBytes(writer, value, size);
    }
    static encodeSignedInt(writer, { value, tag }) {
        const size = this.getSignedIntSize(value);
        this.encodeControlByteAndTag(writer, TlvType.UnsignedInt | size, tag);
        this.encodeSignedIntBytes(writer, value, size);
    }
    static encodeBoolean(writer, { value, tag }) {
        this.encodeControlByteAndTag(writer, TlvType.Boolean | (value ? 1 : 0), tag);
    }
    static encodeContainer(writer, { type, value, tag }) {
        this.encodeControlByteAndTag(writer, type, tag);
        value.forEach(element => this.encodeElementInternal(writer, element));
        this.encodeControlByteAndTag(writer, TlvType.EndOfContainer, TlvTag.Anonymous);
    }
    static encodeControlByteAndTag(writer, valueType, tag) {
        var tagControl;
        var longTag = (tag.id & 0xFFFF0000) !== 0;
        if (tag.isAnonymous()) {
            tagControl = 0;
        }
        else if (tag.isContextual()) {
            tagControl = 32;
        }
        else if (tag.isCommon()) {
            tagControl = longTag ? 96 : 64;
        }
        else {
            tagControl = longTag ? 224 : 192;
        }
        writer.writeUInt8(tagControl | valueType);
        switch (tagControl) {
            case 32:
                writer.writeUInt8(tag.id);
                break;
            case 64:
                writer.writeUInt16(tag.id);
                break;
            case 96:
                writer.writeUInt32(tag.id);
                break;
            case 192:
                writer.writeUInt32(tag.profile);
                writer.writeUInt16(tag.id);
                break;
            case 224:
                writer.writeUInt32(tag.profile);
                writer.writeUInt32(tag.id);
                break;
            case 0:
                break;
        }
    }
    static encodeUnsignedIntBytes(writer, value, size) {
        if (typeof value === "number") {
            switch (size) {
                case 0:
                    writer.writeUInt8(value);
                    break;
                case 1:
                    writer.writeUInt16(value);
                    break;
                case 2:
                    writer.writeUInt32(value);
                    break;
                case 3:
                    throw new Error("64 bits number should be stored on a bigint");
            }
        }
        else {
            switch (size) {
                case 0:
                    writer.writeUInt8(Number(value));
                    break;
                case 1:
                    writer.writeUInt16(Number(value));
                    break;
                case 2:
                    writer.writeUInt32(Number(value));
                    break;
                case 3:
                    writer.writeUInt64(value);
                    break;
            }
        }
    }
    static encodeSignedIntBytes(writer, value, size) {
        if (typeof value === "number") {
            switch (size) {
                case 0:
                    writer.writeInt8(value);
                    break;
                case 1:
                    writer.writeInt16(value);
                    break;
                case 2:
                    writer.writeInt32(value);
                    break;
                case 3:
                    throw new Error("64 bits number should be stored on a bigint");
            }
        }
        else {
            switch (size) {
                case 0:
                    writer.writeInt8(Number(value));
                    break;
                case 1:
                    writer.writeInt16(Number(value));
                    break;
                case 2:
                    writer.writeInt32(Number(value));
                    break;
                case 3:
                    writer.writeInt64(value);
                    break;
            }
        }
    }
    static getUnsignedIntSize(value) {
        if (value < 256) {
            return 0;
        }
        else if (value < 65536) {
            return 1;
        }
        else if (value < 4294967296) {
            return 2;
        }
        else {
            return 3;
        }
    }
    static getSignedIntSize(value) {
        if (value < 128 && value >= -128) {
            return 0;
        }
        else if (value < 32767 && value >= -32768) {
            return 1;
        }
        else if (value < 2147483647 && value >= -2147483648) {
            return 2;
        }
        else {
            return 3;
        }
    }
}
exports.TlvCodec = TlvCodec;
