"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessageExchange = void 0;
const MessageCodec_1 = require("../../codec/MessageCodec");
const Queue_1 = require("../../util/Queue");
const SecureChannelMessages_1 = require("../session/secure/SecureChannelMessages");
const Promises_1 = require("../../util/Promises");
const Time_1 = require("../../time/Time");
const Logger_1 = require("../../log/Logger");
const logger = Logger_1.Logger.get("MessageExchange");
class MessageExchange {
    constructor(session, channel, messageCounter, isInitiator, peerSessionId, nodeId, peerNodeId, exchangeId, protocolId, closeCallback) {
        this.session = session;
        this.channel = channel;
        this.messageCounter = messageCounter;
        this.isInitiator = isInitiator;
        this.peerSessionId = peerSessionId;
        this.nodeId = nodeId;
        this.peerNodeId = peerNodeId;
        this.exchangeId = exchangeId;
        this.protocolId = protocolId;
        this.closeCallback = closeCallback;
        this.messagesQueue = new Queue_1.Queue();
        const { activeRetransmissionTimeoutMs: activeRetransmissionTimeoutMs, retransmissionRetries } = session.getMrpParameters();
        this.activeRetransmissionTimeoutMs = activeRetransmissionTimeoutMs;
        this.retransmissionRetries = retransmissionRetries;
    }
    static fromInitialMessage(channel, messageCounter, initialMessage, closeCallback) {
        const { session } = channel;
        const exchange = new MessageExchange(session, channel, messageCounter, false, session.getId(), initialMessage.packetHeader.destNodeId, initialMessage.packetHeader.sourceNodeId, initialMessage.payloadHeader.exchangeId, initialMessage.payloadHeader.protocolId, closeCallback);
        exchange.onMessageReceived(initialMessage);
        return exchange;
    }
    static initiate(channel, exchangeId, protocolId, messageCounter, closeCallback) {
        const { session } = channel;
        return new MessageExchange(session, channel, messageCounter, true, session.getPeerSessionId(), session.getNodeId(), session.getPeerNodeId(), exchangeId, protocolId, closeCallback);
    }
    onMessageReceived(message) {
        var _a, _b, _c, _d, _e;
        const { packetHeader: { messageId }, payloadHeader: { requiresAck, ackedMessageId, protocolId, messageType } } = message;
        logger.debug("onMessageReceived", MessageCodec_1.MessageCodec.messageToString(message));
        logger.debug(this.exchangeId, this.sentMessageToAck !== undefined ? MessageCodec_1.MessageCodec.messageToString(this.sentMessageToAck) : "no message to ack");
        if (messageId === ((_a = this.receivedMessageToAck) === null || _a === void 0 ? void 0 : _a.packetHeader.messageId)) {
            if (requiresAck) {
                this.send(16, Buffer.alloc(0));
            }
            return;
        }
        if (messageId === ((_b = this.sentMessageToAck) === null || _b === void 0 ? void 0 : _b.payloadHeader.ackedMessageId)) {
            this.channel.send(this.sentMessageToAck);
            return;
        }
        const sentMessageIdToAck = (_c = this.sentMessageToAck) === null || _c === void 0 ? void 0 : _c.packetHeader.messageId;
        if (sentMessageIdToAck !== undefined) {
            if (ackedMessageId === undefined)
                throw new Error("Previous message ack is missing");
            if (ackedMessageId !== sentMessageIdToAck)
                throw new Error(`Incorrect ack received. Expected ${sentMessageIdToAck}, received: ${ackedMessageId}`);
            (_d = this.sentMessageAckSuccess) === null || _d === void 0 ? void 0 : _d.call(this);
            this.sentMessageToAck = undefined;
            (_e = this.retransmissionTimer) === null || _e === void 0 ? void 0 : _e.stop();
        }
        if (protocolId === SecureChannelMessages_1.SECURE_CHANNEL_PROTOCOL_ID && messageType === 16) {
            return;
        }
        if (protocolId !== this.protocolId) {
            throw new Error(`Received a message for an unexpected protocol. Expected: ${this.protocolId}, received: ${protocolId}`);
        }
        if (requiresAck) {
            this.receivedMessageToAck = message;
        }
        this.messagesQueue.write(message);
    }
    send(messageType, payload) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            if (this.sentMessageToAck !== undefined)
                throw new Error("The previous message has not been acked yet, cannot send a new message");
            const message = {
                packetHeader: {
                    sessionId: this.peerSessionId,
                    sessionType: 0,
                    messageId: this.messageCounter.getIncrementedCounter(),
                    destNodeId: this.peerNodeId,
                    sourceNodeId: this.nodeId,
                },
                payloadHeader: {
                    exchangeId: this.exchangeId,
                    protocolId: messageType === 16 ? SecureChannelMessages_1.SECURE_CHANNEL_PROTOCOL_ID : this.protocolId,
                    messageType,
                    isInitiatorMessage: this.isInitiator,
                    requiresAck: messageType === 16 ? false : true,
                    ackedMessageId: (_a = this.receivedMessageToAck) === null || _a === void 0 ? void 0 : _a.packetHeader.messageId,
                },
                payload,
            };
            this.receivedMessageToAck = undefined;
            let ackPromise;
            if (message.payloadHeader.requiresAck) {
                this.sentMessageToAck = message;
                this.retransmissionTimer = Time_1.Time.getTimer(this.activeRetransmissionTimeoutMs, () => this.retransmitMessage(message, 1))
                    .start();
                const { promise, resolver, rejecter } = yield (0, Promises_1.getPromiseResolver)();
                this.sentMessageAckSuccess = resolver;
                this.sentMessageAckFailure = rejecter;
            }
            yield this.channel.send(message);
            if (ackPromise !== undefined) {
                yield ackPromise;
                this.sentMessageAckFailure = undefined;
                this.sentMessageAckFailure = undefined;
            }
        });
    }
    nextMessage() {
        return this.messagesQueue.read();
    }
    waitFor(messageType) {
        return __awaiter(this, void 0, void 0, function* () {
            const message = yield this.messagesQueue.read();
            const { payloadHeader: { messageType: receivedMessageType } } = message;
            if (receivedMessageType !== messageType)
                throw new Error(`Received unexpected message type ${receivedMessageType.toString(16)}. Expected ${messageType.toString(16)}`);
            return message;
        });
    }
    retransmitMessage(message, retransmissionCount) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.channel.send(message);
            retransmissionCount++;
            if (retransmissionCount === this.retransmissionRetries)
                return;
            this.retransmissionTimer = Time_1.Time.getTimer(this.activeRetransmissionTimeoutMs, () => this.retransmitMessage(message, retransmissionCount))
                .start();
        });
    }
    close() {
        if (this.receivedMessageToAck !== undefined) {
            this.send(16, Buffer.alloc(0));
        }
        Time_1.Time.getTimer(this.activeRetransmissionTimeoutMs * (this.retransmissionRetries + 1), () => this.closeInternal())
            .start();
    }
    closeInternal() {
        var _a, _b;
        (_a = this.retransmissionTimer) === null || _a === void 0 ? void 0 : _a.stop();
        (_b = this.sentMessageAckFailure) === null || _b === void 0 ? void 0 : _b.call(this);
        this.messagesQueue.close();
        this.closeCallback();
    }
}
exports.MessageExchange = MessageExchange;
