"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FabricBuilder = exports.Fabric = void 0;
const TlvObjectCodec_1 = require("../../codec/TlvObjectCodec");
const Crypto_1 = require("../../crypto/Crypto");
const LEBufferWriter_1 = require("../../util/LEBufferWriter");
const CertificateManager_1 = require("../certificate/CertificateManager");
const COMPRESSED_FABRIC_ID_INFO = Buffer.from("CompressedFabric");
const GROUP_SECURITY_INFO = Buffer.from("GroupKey v1.0");
class Fabric {
    constructor(id, nodeId, operationalId, rootPublicKey, keyPair, vendorId, rootCert, identityProtectionKey, operationalIdentityProtectionKey, intermediateCACert, operationalCert) {
        this.id = id;
        this.nodeId = nodeId;
        this.operationalId = operationalId;
        this.rootPublicKey = rootPublicKey;
        this.keyPair = keyPair;
        this.vendorId = vendorId;
        this.rootCert = rootCert;
        this.identityProtectionKey = identityProtectionKey;
        this.operationalIdentityProtectionKey = operationalIdentityProtectionKey;
        this.intermediateCACert = intermediateCACert;
        this.operationalCert = operationalCert;
    }
    getPublicKey() {
        return this.keyPair.publicKey;
    }
    sign(data) {
        return Crypto_1.Crypto.sign(this.keyPair.privateKey, data);
    }
    verifyCredentials(operationalCert, intermediateCACert) {
        return;
    }
    getDestinationId(nodeId, random) {
        const writter = new LEBufferWriter_1.LEBufferWriter();
        writter.writeBytes(random);
        writter.writeBytes(this.rootPublicKey);
        writter.writeUInt64(this.id);
        writter.writeUInt64(nodeId);
        const elements = writter.toBuffer();
        return Crypto_1.Crypto.hmac(this.operationalIdentityProtectionKey, elements);
    }
}
exports.Fabric = Fabric;
class FabricBuilder {
    constructor() {
        this.keyPair = Crypto_1.Crypto.createKeyPair();
    }
    getPublicKey() {
        return this.keyPair.publicKey;
    }
    createCertificateSigningRequest() {
        return CertificateManager_1.CertificateManager.createCertificateSigningRequest(this.keyPair);
    }
    setRootCert(rootCert) {
        this.rootCert = rootCert;
        this.rootPublicKey = TlvObjectCodec_1.TlvObjectCodec.decode(rootCert, CertificateManager_1.RootCertificateT).ellipticCurvePublicKey;
        return this;
    }
    setOperationalCert(operationalCert) {
        this.operationalCert = operationalCert;
        const { subject: { nodeId, fabricId } } = TlvObjectCodec_1.TlvObjectCodec.decode(operationalCert, CertificateManager_1.OperationalCertificateT);
        this.fabricId = fabricId;
        this.nodeId = nodeId;
        return this;
    }
    setIntermediateCACert(certificate) {
        this.intermediateCACert = certificate;
        return this;
    }
    setVendorId(vendorId) {
        this.vendorId = vendorId;
        return this;
    }
    setIdentityProtectionKey(key) {
        this.identityProtectionKey = key;
        return this;
    }
    build() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.vendorId === undefined)
                throw new Error("vendorId needs to be set");
            if (this.rootCert === undefined || this.rootPublicKey === undefined)
                throw new Error("rootCert needs to be set");
            if (this.identityProtectionKey === undefined)
                throw new Error("identityProtectionKey needs to be set");
            if (this.operationalCert === undefined || this.fabricId === undefined || this.nodeId === undefined)
                throw new Error("operationalCert needs to be set");
            const operationalIdSalt = Buffer.alloc(8);
            operationalIdSalt.writeBigUInt64BE(BigInt(this.fabricId));
            const operationalId = yield Crypto_1.Crypto.hkdf(this.rootPublicKey.slice(1), operationalIdSalt, COMPRESSED_FABRIC_ID_INFO, 8);
            return new Fabric(this.fabricId, this.nodeId, operationalId, this.rootPublicKey, this.keyPair, this.vendorId, this.rootCert, this.identityProtectionKey, yield Crypto_1.Crypto.hkdf(this.identityProtectionKey, operationalId, GROUP_SECURITY_INFO, 16), this.intermediateCACert, this.operationalCert);
        });
    }
}
exports.FabricBuilder = FabricBuilder;
