"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.InteractionClientMessenger = exports.InteractionServerMessenger = void 0;
const TlvObjectCodec_1 = require("../../codec/TlvObjectCodec");
const Logger_1 = require("../../log/Logger");
const InteractionMessages_1 = require("./InteractionMessages");
const logger = Logger_1.Logger.get("InteractionServerMessenger");
class InteractionMessenger {
    constructor(exchangeBase) {
        this.exchangeBase = exchangeBase;
    }
    sendStatus(status) {
        return this.exchangeBase.send(1, TlvObjectCodec_1.TlvObjectCodec.encode({ status, interactionModelRevision: 1 }, InteractionMessages_1.StatusResponseT));
    }
    waitForSuccess() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.nextMessage(1);
        });
    }
    nextMessage(expectedMessageType) {
        return __awaiter(this, void 0, void 0, function* () {
            const message = yield this.exchangeBase.nextMessage();
            const messageType = message.payloadHeader.messageType;
            this.throwIfError(messageType, message.payload);
            if (expectedMessageType !== undefined && messageType !== expectedMessageType)
                throw new Error(`Received unexpected message type: ${messageType}, expected: ${expectedMessageType}`);
            return message;
        });
    }
    close() {
        this.exchangeBase.close();
    }
    throwIfError(messageType, payload) {
        if (messageType !== 1)
            return;
        const { status } = TlvObjectCodec_1.TlvObjectCodec.decode(payload, InteractionMessages_1.StatusResponseT);
        if (status !== 0)
            new Error(`Received error status: ${status}`);
    }
}
class InteractionServerMessenger extends InteractionMessenger {
    constructor(exchange) {
        super(exchange);
        this.exchange = exchange;
    }
    handleRequest(handleReadRequest, handleSubscribeRequest, handleInvokeRequest) {
        return __awaiter(this, void 0, void 0, function* () {
            const message = yield this.exchange.nextMessage();
            try {
                switch (message.payloadHeader.messageType) {
                    case 2:
                        const readRequest = TlvObjectCodec_1.TlvObjectCodec.decode(message.payload, InteractionMessages_1.ReadRequestT);
                        yield this.sendDataReport(handleReadRequest(readRequest));
                        break;
                    case 3:
                        const subscribeRequest = TlvObjectCodec_1.TlvObjectCodec.decode(message.payload, InteractionMessages_1.SubscribeRequestT);
                        const subscribeResponse = handleSubscribeRequest(subscribeRequest);
                        if (subscribeRequest === undefined) {
                            yield this.sendStatus(0);
                        }
                        else {
                            yield this.exchange.send(4, TlvObjectCodec_1.TlvObjectCodec.encode(subscribeResponse, InteractionMessages_1.SubscribeResponseT));
                        }
                        break;
                    case 8:
                        const invokeRequest = TlvObjectCodec_1.TlvObjectCodec.decode(message.payload, InteractionMessages_1.InvokeRequestT);
                        const invokeResponse = yield handleInvokeRequest(invokeRequest);
                        yield this.exchange.send(9, TlvObjectCodec_1.TlvObjectCodec.encode(invokeResponse, InteractionMessages_1.InvokeResponseT));
                        break;
                    default:
                        throw new Error(`Unsupported message type ${message.payloadHeader.messageType}`);
                }
            }
            catch (error) {
                logger.error(error);
                yield this.sendStatus(1);
            }
            finally {
                this.exchange.close();
            }
        });
    }
    sendDataReport(dataReport) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.exchange.send(5, TlvObjectCodec_1.TlvObjectCodec.encode(dataReport, InteractionMessages_1.DataReportT));
        });
    }
}
exports.InteractionServerMessenger = InteractionServerMessenger;
class InteractionClientMessenger extends InteractionMessenger {
    constructor(exchange) {
        super(exchange);
        this.exchange = exchange;
    }
    sendReadRequest(readRequest) {
        return this.request(2, InteractionMessages_1.ReadRequestT, 5, InteractionMessages_1.DataReportT, readRequest);
    }
    sendSubscribeRequest(subscribeRequest) {
        return this.request(3, InteractionMessages_1.SubscribeRequestT, 4, InteractionMessages_1.SubscribeResponseT, subscribeRequest);
    }
    sendInvokeCommand(invokeRequest) {
        return this.request(8, InteractionMessages_1.InvokeRequestT, 9, InteractionMessages_1.InvokeResponseT, invokeRequest);
    }
    readDataReport() {
        return __awaiter(this, void 0, void 0, function* () {
            const dataReportMessage = yield this.exchange.waitFor(5);
            return TlvObjectCodec_1.TlvObjectCodec.decode(dataReportMessage.payload, InteractionMessages_1.DataReportT);
        });
    }
    request(requestMessageType, requestTemplate, responseMessageType, responseTemplate, request) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.exchange.send(requestMessageType, TlvObjectCodec_1.TlvObjectCodec.encode(request, requestTemplate));
            const responseMessage = yield this.nextMessage(responseMessageType);
            return TlvObjectCodec_1.TlvObjectCodec.decode(responseMessage.payload, responseTemplate);
        });
    }
}
exports.InteractionClientMessenger = InteractionClientMessenger;
