"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.InteractionServer = exports.pathToId = exports.ClusterServer = exports.INTERACTION_PROTOCOL_ID = void 0;
const InteractionMessenger_1 = require("./InteractionMessenger");
const CommandServer_1 = require("../cluster/server/CommandServer");
const DescriptorCluster_1 = require("../cluster/DescriptorCluster");
const TlvObjectCodec_1 = require("../../codec/TlvObjectCodec");
const AttributeServer_1 = require("../cluster/server/AttributeServer");
const SubscriptionHandler_1 = require("./SubscriptionHandler");
const Logger_1 = require("../../log/Logger");
exports.INTERACTION_PROTOCOL_ID = 0x0001;
class ClusterServer {
    constructor(clusterDef, attributesInitialValues, handlers) {
        this.attributes = {};
        this.commands = new Array();
        const { id, attributes: attributeDefs, commands: commandDefs } = clusterDef;
        this.id = id;
        for (const name in attributesInitialValues) {
            const { id, template } = attributeDefs[name];
            this.attributes[name] = new AttributeServer_1.AttributeServer(id, name, template, attributesInitialValues[name]);
        }
        for (const name in commandDefs) {
            const handler = handlers[name];
            if (handler === undefined)
                continue;
            const { requestId, requestTemplate, responseId, responseTemplate } = commandDefs[name];
            this.commands.push(new CommandServer_1.CommandServer(requestId, responseId, name, requestTemplate, responseTemplate, (request, session) => handler({ request, attributes: this.attributes, session })));
        }
    }
}
exports.ClusterServer = ClusterServer;
function pathToId({ endpointId, clusterId, id }) {
    return `${endpointId}/${clusterId}/${id}`;
}
exports.pathToId = pathToId;
const logger = Logger_1.Logger.get("InteractionProtocol");
class InteractionServer {
    constructor() {
        this.attributes = new Map();
        this.attributePaths = new Array();
        this.commands = new Map();
        this.commandPaths = new Array();
    }
    getId() {
        return exports.INTERACTION_PROTOCOL_ID;
    }
    addEndpoint(endpointId, device, clusters) {
        const descriptorCluster = new ClusterServer(DescriptorCluster_1.DescriptorCluster, {
            deviceTypeList: [{ revision: 1, type: device.code }],
            serverList: [],
            clientList: [],
            partsList: [],
        }, {});
        clusters.push(descriptorCluster);
        descriptorCluster.attributes.serverList.set(clusters.map(({ id }) => id));
        clusters.forEach(({ id: clusterId, attributes, commands }) => {
            for (const name in attributes) {
                const attribute = attributes[name];
                const path = { endpointId, clusterId, id: attribute.id };
                this.attributes.set(pathToId(path), attribute);
                this.attributePaths.push(path);
            }
            commands.forEach(command => {
                const path = { endpointId, clusterId, id: command.invokeId };
                this.commands.set(pathToId(path), command);
                this.commandPaths.push(path);
            });
        });
        if (endpointId !== 0) {
            const rootPartsListAttribute = this.attributes.get(pathToId({ endpointId: 0, clusterId: DescriptorCluster_1.DescriptorCluster.id, id: DescriptorCluster_1.DescriptorCluster.attributes.partsList.id }));
            if (rootPartsListAttribute === undefined)
                throw new Error("The root endpoint should be added first!");
            rootPartsListAttribute.set([...rootPartsListAttribute.get(), endpointId]);
        }
        return this;
    }
    onNewExchange(exchange) {
        return __awaiter(this, void 0, void 0, function* () {
            yield new InteractionMessenger_1.InteractionServerMessenger(exchange).handleRequest(readRequest => this.handleReadRequest(exchange, readRequest), subscribeRequest => this.handleSubscribeRequest(exchange, subscribeRequest), invokeRequest => this.handleInvokeRequest(exchange, invokeRequest));
        });
    }
    handleReadRequest(exchange, { attributes: attributePaths }) {
        logger.debug(`Received read request from ${exchange.channel.getName()}: ${attributePaths.map(({ endpointId = "*", clusterId = "*", id = "*" }) => `${endpointId}/${clusterId}/${id}`).join(", ")}`);
        const values = this.getAttributes(attributePaths)
            .map(({ path, attribute }) => {
            const { value, version } = attribute.getWithVersion();
            return { path, value, version, template: attribute.template };
        });
        return {
            isFabricFiltered: true,
            interactionModelRevision: 1,
            values: values.map(({ path, value, version, template }) => ({
                value: {
                    path,
                    version,
                    value: TlvObjectCodec_1.TlvObjectCodec.encodeElement(value, template),
                },
            })),
        };
    }
    handleSubscribeRequest(exchange, { minIntervalFloorSeconds, maxIntervalCeilingSeconds, attributeRequests, keepSubscriptions }) {
        logger.debug(`Received subscribe request from ${exchange.channel.getName()}`);
        if (!exchange.session.isSecure())
            throw new Error("Subscriptions are only implemented on secure sessions");
        const session = exchange.session;
        const fabric = session.getFabric();
        if (fabric === undefined)
            throw new Error("Subscriptions are only implemented after a fabric has been assigned");
        if (!keepSubscriptions) {
            session.clearSubscriptions();
        }
        if (attributeRequests !== undefined) {
            const attributes = this.getAttributes(attributeRequests);
            if (attributeRequests.length === 0)
                throw new Error("Invalid subscription request");
            if (minIntervalFloorSeconds < 0)
                throw new Error("minIntervalFloorSeconds should be greater or equal to 0");
            if (maxIntervalCeilingSeconds < 0)
                throw new Error("maxIntervalCeilingSeconds should be greater or equal to 1");
            if (maxIntervalCeilingSeconds < minIntervalFloorSeconds)
                throw new Error("maxIntervalCeilingSeconds should be greater or equal to minIntervalFloorSeconds");
            const subscriptionId = session.addSubscription(SubscriptionHandler_1.SubscriptionHandler.Builder(session.getContext(), fabric, session.getPeerNodeId(), attributes, minIntervalFloorSeconds, maxIntervalCeilingSeconds));
            return { subscriptionId, maxIntervalCeilingSeconds, interactionModelRevision: 1 };
        }
    }
    handleInvokeRequest(exchange, { invokes }) {
        return __awaiter(this, void 0, void 0, function* () {
            logger.debug(`Received invoke request from ${exchange.channel.getName()}: ${invokes.map(({ path: { endpointId, clusterId, id } }) => `${endpointId}/${clusterId}/${id}`).join(", ")}`);
            const results = new Array();
            yield Promise.all(invokes.map(({ path, args }) => __awaiter(this, void 0, void 0, function* () {
                const command = this.commands.get(pathToId(path));
                if (command === undefined)
                    return;
                const result = yield command.invoke(exchange.session, args);
                results.push(Object.assign(Object.assign({}, result), { path }));
            })));
            return {
                suppressResponse: false,
                interactionModelRevision: 1,
                responses: results.map(({ path, responseId, code, response }) => {
                    if (response === undefined) {
                        return { result: { path, result: { code } } };
                    }
                    else {
                        return { response: { path: Object.assign(Object.assign({}, path), { id: responseId }), response } };
                    }
                }),
            };
        });
    }
    getAttributes(filters) {
        const result = new Array();
        filters.forEach(({ endpointId, clusterId, id }) => {
            if (endpointId !== undefined && clusterId !== undefined && id !== undefined) {
                const path = { endpointId, clusterId, id };
                const attribute = this.attributes.get(pathToId(path));
                if (attribute === undefined)
                    return;
                result.push({ path, attribute });
            }
            else {
                this.attributePaths.filter(path => (endpointId === undefined || endpointId === path.endpointId)
                    && (clusterId === undefined || clusterId === path.clusterId)
                    && (id === undefined || id === path.id))
                    .forEach(path => result.push({ path, attribute: this.attributes.get(pathToId(path)) }));
            }
        });
        return result;
    }
}
exports.InteractionServer = InteractionServer;
