"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MdnsServer = exports.MDNS_BROADCAST_PORT = exports.MDNS_BROADCAST_IP = void 0;
const DnsCodec_1 = require("../codec/DnsCodec");
const Network_1 = require("./Network");
const UdpMulticastServer_1 = require("./UdpMulticastServer");
const Cache_1 = require("../util/Cache");
exports.MDNS_BROADCAST_IP = "224.0.0.251";
exports.MDNS_BROADCAST_PORT = 5353;
class MdnsServer {
    constructor(multicastInterface, multicastServer) {
        this.multicastInterface = multicastInterface;
        this.multicastServer = multicastServer;
        this.network = Network_1.Network.get();
        this.recordsGenerator = () => [];
        this.records = new Cache_1.Cache((ip, mac) => this.recordsGenerator(ip, mac), 5 * 60 * 1000);
        multicastServer.onMessage((message, remoteIp) => this.handleDnsMessage(message, remoteIp));
    }
    static create(multicastInterface) {
        return __awaiter(this, void 0, void 0, function* () {
            return new MdnsServer(multicastInterface, yield UdpMulticastServer_1.UdpMulticastServer.create({ multicastInterface, broadcastAddress: exports.MDNS_BROADCAST_IP, listeningPort: exports.MDNS_BROADCAST_PORT }));
        });
    }
    handleDnsMessage(messageBytes, remoteIp) {
        const ipMac = this.network.getIpMacOnInterface(remoteIp);
        if (ipMac === undefined)
            return;
        const { ip, mac } = ipMac;
        const records = this.records.get(ip, mac);
        if (records.length === 0)
            return;
        const message = DnsCodec_1.DnsCodec.decode(messageBytes);
        if (message === undefined)
            return;
        if (message.messageType !== 0)
            return;
        const answers = message.queries.flatMap(query => this.queryRecords(query, records));
        if (answers.length === 0)
            return;
        const additionalRecords = records.filter(record => !answers.includes(record));
        this.multicastServer.send(DnsCodec_1.DnsCodec.encode({ answers, additionalRecords }), ip);
    }
    announce() {
        return __awaiter(this, void 0, void 0, function* () {
            yield Promise.all(this.getIpMacsForAnnounce().map(({ ip, mac }) => {
                const records = this.records.get(ip, mac);
                const answers = records.filter(({ recordType }) => recordType === 12);
                const additionalRecords = records.filter(({ recordType }) => recordType !== 12);
                return this.multicastServer.send(DnsCodec_1.DnsCodec.encode({ answers, additionalRecords }), ip);
            }));
        });
    }
    setRecordsGenerator(generator) {
        this.records.clear();
        this.recordsGenerator = generator;
    }
    close() {
        this.records.close();
        this.multicastServer.close();
    }
    getIpMacsForAnnounce() {
        if (this.multicastInterface === undefined)
            return this.network.getIpMacAddresses();
        const ipMac = this.network.getIpMacOnInterface(this.multicastInterface);
        if (ipMac === undefined)
            return [];
        return [ipMac];
    }
    queryRecords({ name, recordType }, records) {
        if (recordType === 255) {
            return records.filter(record => record.name === name);
        }
        else {
            return records.filter(record => record.name === name && record.recordType === recordType);
        }
    }
}
exports.MdnsServer = MdnsServer;
